from time import time, sleep

from rdflib.syntax.parsers import RDFXMLParser
from rdflib.TripleStore import TripleStore
from rdflib.RDF import RDFNS

import rdflib
import urllib

class Downloader:
  """A rate-limited downloader, with knowledge of del.icio.us URLs and
   a thin wrapper around RDFlib."""
  def __init__(self):
    self.nextAttempt = None
    self.urlOpener = urllib.FancyURLopener()
    self.urlOpener.http_error_default = self.http_error_default

  def http_error_default(self, url, fp, errcode, errmsg, headers):
    raise 'Download failed for %s: %d %s' % (url, errcode, errmsg)

  def _pause(self):
    while True:
      t = time()
      if t >= self.nextAttempt:
        break
      else:
        sleep(self.nextAttempt - t)
    self.nextAttempt = t + 1

  def _downloadToStore(self,url):
    s = TripleStore()
    p = RDFXMLParser.RDFXMLParser(s)
    h = self.urlOpener.open(url)
    p.parse(h)
    return s

  def download(self, url):
    self._pause()
    return self._downloadToStore(url)

  def fetchUserRdf(self, username):
    return self.download('http://del.icio.us/rss/' + username)

  def fetchUrlRdf(self, url):
    return self.download('http://del.icio.us/rss/url/?url=' + urllib.quote(url))

def seqItems(store,seq):
  i = 1
  while True:
    x = rdflib.util.first(store.objects(seq, RDFNS['_' + str(i)]))
    if not(x):
	  return
    yield x
    i += 1
