#!/usr/bin/python

"""
Check a del.icio.us user's bookmarks, and graph how well they have
predicted popular URLs.
"""

from sys import exit
from sys import argv, stderr, path, stdout

if len(path) > 0:
  import os.path
  path.insert(0, os.path.join(path[0], 'rdflib-2.0.6'))

from rdflib.Namespace import Namespace
from rdflib.URIRef import URIRef
from rdflib.util import parse_date_time
import rdflib

from xml.sax._exceptions import SAXException

from ddl import Downloader, seqItems

if len(argv) < 3:
  print >>stderr,"Usage:",argv[0],"<user> <filename.png> [title]"
  exit(5)

user = argv[1]

def parseISO(s):
  if len(s) == 17:
    s = s[:16] + ':00Z'
  return parse_date_time(s)

g = None
try:
  import graph
  g = graph.Graph(argv[2])
except:
  print >>stderr,"Graphing not available (see graph.py)"

d = Downloader()

RSS = Namespace('http://purl.org/rss/1.0/')
DC = Namespace('http://purl.org/dc/elements/1.1/')

print >>stderr,"Downloading history for",user,"..."
s = d.fetchUserRdf(user)

items = rdflib.util.first(s.objects(URIRef('http://del.icio.us/' + user), RSS['items']))
if not(items):
  print >>stderr, 'No RSS items found'
  exit(10)

ticks={}

def tick(t,v):
  if t in ticks:
    ticks[t] += v
  else:
    ticks[t] = v

def stampedDetails():
  for x in seqItems(s, items):
    buyDate = rdflib.util.first(s.objects(x, DC['date']))
    if not(buyDate):
      print >>stderr,'No date for',x
      exit(10)

    buyDate = parseISO(buyDate)

    stderr.write(x +' ... ')
    try:
      i = d.fetchUrlRdf(x)
      # Has this overflowed?
      if len([z for z in i.objects(x, DC['creator'])]) == 30:
        print >>stderr,"(maxed out)"
      else:
        yield (buyDate, x, i)
    except (IOError, SAXException), e:
      print >>stderr,"Failed to parse:",e

for (buyDate, x, i) in stampedDetails():
  countBefore = 0
  countAfter = 0

  for o in i.objects(x, DC['date']):
    t = parseISO(o)
    if t <= buyDate:
      countBefore += 1
    else:
      countAfter += 1
      tick(t, 1);

  tick(buyDate, -countBefore)

  print >>stderr,'(-%d, +%d)' % (countBefore, countAfter)

l = ticks.keys()
l.sort()

p = 0
for t in l:
  p += ticks[t]
  if g:
    g.add(t, p)

if g:
  if len(argv) == 4:
    g.end(argv[3])
  else:
    g.end('del.icio.us/' + user)

print "Final balance:",p
